package llm

import (
	"context"

	"github.com/tmc/langchaingo/llms"
	"github.com/tmc/langchaingo/llms/ollama"
)

var LLMGroups map[string]*LLMGroup
var LLMNames []string

// const ollamaurl = "http://192.168.1.5:11434"
const ollamaurl = "http://ishangsf.com:11434"

func init() {
	models := []string{"qwen2.5-coder:7b", "deepseek-coder-v2:16b"}
	LLMNames = make([]string, len(models))
	LLMGroups = make(map[string]*LLMGroup, len(models))

	for i, model := range models {
		llm, err := ollama.New(ollama.WithServerURL(ollamaurl), ollama.WithModel(model))
		if err != nil {
			panic(err)
		}
		model = "local " + model
		g := &LLMGroup{
			name: model,
			llm: make(chan llms.Model, 1),
			local: true,
		}
		g.llm <- llm
		LLMGroups[model] = g
		LLMNames[i] = model
	}
}



type LLMGroup struct {
	name  string
	llm   chan llms.Model
	local bool
}

func (g *LLMGroup) Name() string { return g.name }
func (g *LLMGroup) IsLocal() bool { return g.local }
func (g *LLMGroup) Cap() int { return cap(g.llm) }

func (g *LLMGroup) GenerateContent(ctx context.Context, messages []llms.MessageContent, options ...llms.CallOption) (*llms.ContentResponse, error) {
	m := <-g.llm
	defer func(){
		g.llm <- m
	}()
	return m.GenerateContent(ctx, messages, options...)
}

// use streaming if stream is not nil
func (g *LLMGroup) Chat(ctx context.Context, msg string, options ...llms.CallOption) (string, error) {
	contents := []llms.MessageContent{llms.TextParts(llms.ChatMessageTypeHuman, msg)}
	return g.ChatContents(ctx, contents, options...)
}

// use streaming if stream is not nil
func (g *LLMGroup) ChatContents(ctx context.Context, contents []llms.MessageContent, options ...llms.CallOption) (string, error) {
	m := <-g.llm
	defer func(){
		g.llm <- m
	}()

	options = append(options, llms.WithN(1))
	res, err := m.GenerateContent(ctx, contents, options...)
	if err != nil {
		return "", err
	}
	if len(res.Choices) == 0 {
		return "", nil
	}
	return res.Choices[0].Content, nil
}

// block util available
func (g LLMGroup) Get() llms.Model {
	return <-g.llm
}

func ContainLLM(name string) bool {
	_, ok := LLMGroups[name]
	return ok
}
